from ..client import Client, Scope
from typing import List, Dict, Optional, Any
from enum import Enum
from .ad_error_handler import catch_exception, ADSuccessResult

class AccountControlCode(Enum):
    """Possible Account Control States.
    ref <https://learn.microsoft.com/en-us/troubleshoot/windows-server/identity/useraccountcontrol-manipulate-account-properties>
    """

    NormalAccount = 512
    DisableAccount = 514
        
    def __repr__(self) -> str:
        return f'AccountControl: {self.name}'

def match_code(code: int) -> AccountControlCode:
        """Match the possible code, return Normal Account variant if code doens't matchs any of them."""
        match code:
            case 512:
                return AccountControlCode.NormalAccount
            case 514:
                return AccountControlCode.DisableAccount
            case _:
                return AccountControlCode.NormalAccount


class MsAD(Client):
    """Active Directory client."""

    def __init__(self, uri: str, base_dn: str, bind_dn: str, auth_pass: str, debug: bool = False) -> None:
        Client.__init__(self, uri, 0, debug)
        self.uri = uri
        self.base_dn = base_dn
        self.auth_pass = auth_pass
        self.bind_dn = bind_dn

    @catch_exception
    def connect(self) -> ADSuccessResult:
        """Should connect to ad server."""
        return ADSuccessResult(f'connect::{self._bind(self.bind_dn, self.auth_pass).unwrap()}')

    @catch_exception
    def start_tls(self, ca_path: Optional[str] = None) -> ADSuccessResult:
        """Should start tls over created connection in Active Directory."""
        return ADSuccessResult(f'tls::{self._start_tls(ca_path).unwrap()}')

    @catch_exception
    def close(self) -> ADSuccessResult:
        """Should unbind a created connection in Active Directory."""
        return ADSuccessResult(f'close::{self._close().unwrap()}')

    @catch_exception
    def create_user(
            self, user_name: str,
            user_password: str,
            acc: Optional[AccountControlCode],
            exp: int = 0,
            lockout: int = 0) -> ADSuccessResult:
        """Should create a new user in Active Directory."""
        names = [n.capitalize() for n in user_name.split()]
        name = names[0]
        sn = " ".join(names[1:])
        parsed_dn = self._parse_dn(self.base_dn).unwrap()
        principal_name = f"{name}@{parsed_dn[0][0][1]}.{parsed_dn[1][0][1]}"

        target_dn = f"CN={user_name},CN=Users,{self.base_dn}"
        entry = {}
        entry['objectclass'] = [b'top', b'person', b'organizationalPerson', b'user']
        entry['cn'] = user_name.encode('utf-8')
        entry['sn'] = sn.encode('utf-8')
        entry['unicodePwd'] = f'"{user_password}"'.encode('utf-16-le')
        entry['sAMAccountName'] = name.encode('utf-8')
        entry['givenName'] = name.encode('utf-8')
        entry['userPrincipalName'] = principal_name.encode('utf-8')
        entry['displayName'] = user_name.encode('utf-8')
    
        if acc is not None and type(acc) == AccountControlCode:
            entry['userAccountControl'] = f'{acc.value}'.encode('utf-8')
        else:
            entry['userAccountControl'] = f'{AccountControlCode.NormalAccount.value}'.encode('utf-8')

        entry['lockoutTime'] = f'{lockout}'.encode('utf-8')
        entry['accountExpires'] = f'{exp}'.encode('utf-8')

        return ADSuccessResult(f'create_user::{self._add(target_dn, entry).unwrap()}')

    @catch_exception
    def delete_user(self, user_name: str) -> ADSuccessResult:
        """Should delete user by his username."""
        target_dn = f"CN={user_name},CN=Users,{self.base_dn}"

        return ADSuccessResult(f'delete_user::{self._delete(target_dn).unwrap()}')

    def __get_entries(self, s_filter: str, attr: List[str], scope: Scope=Scope.SubTree) -> List[Dict[Any, Any]]:
        """Get attributes entries by permforming ldap search over a specified filter,base and scope."""
        raw_entry = self._search(self.base_dn, s_filter, attr, scope).unwrap()

        entry = list()

        assert_dict = lambda x: type(x) is dict and x

        for _,val in raw_entry:
            value = assert_dict(val)
            if value:
                entry.append(value)        

        assert len(entry) >= 1, f'__get_entries: {s_filter} not found any entry'
        return entry

    @catch_exception
    def modify_account_control(self, s_filter: str, state: AccountControlCode) -> ADSuccessResult:
        """Should modify user account control."""

        user_entry = self.__get_entries(s_filter, ['distinguishedName', 'userAccountControl'])[0]
        target_dn = user_entry['distinguishedName'][0].decode()
        old_acc = {'userAccountControl': user_entry['userAccountControl']}
        new_acc = {'userAccountControl': [f'{state.value}'.encode()]}
        
        entry = (old_acc, new_acc)
        mod =  self._modify_replace(target_dn, entry).unwrap()

        return ADSuccessResult(f'modify_account_control::{mod}')
    
    @catch_exception
    def create_user_from_ldif(self, ldif_path: str) -> ADSuccessResult:
        """Should create a new user from ldif file, if exists unicodePwd this function will encoded it."""
        all_records = self._parse_ldif(ldif_path).unwrap()
        unicode_pwd = all_records[0][1].get('unicodePwd')
        if unicode_pwd:
            enc = f'"{unicode_pwd[0].decode()}"'.encode('utf-16-le')
            all_records[0][1].update({'unicodePwd': [enc]})

        add = self._add(all_records[0][0], all_records[0][1]).unwrap()

        return ADSuccessResult(f'create_user_from_ldif::{add}')

    @catch_exception
    def add_account_to_group(self, s_filter, group_dn) -> ADSuccessResult:
        """Should add user account to AD group."""
        user_entry = self.__get_entries(s_filter, ['distinguishedName'])[0]
        user_dn = user_entry['distinguishedName'][0]
        entry = {'member': [user_dn]}
        mod = self._modify_add(group_dn, entry).unwrap()

        return ADSuccessResult(f'add_account_to_group::{mod}, user: {str(user_dn)}')

    @catch_exception
    def remove_account_from_group(self, s_filter: str, group_dn: str) -> ADSuccessResult:
        """Should remove user account from a group of ms ad."""
        user_entry = self.__get_entries(s_filter, ['distinguishedName'])[0]
        user_dn = user_entry['distinguishedName'][0]
        entry = {'member': [user_dn]}
        mod = self._modify_delete(group_dn, entry).unwrap()

        return ADSuccessResult(f'remove_account_from_group::{mod} user: {user_dn.decode()}')

    def __str__(self) -> str:
        return f'uri: {self.uri}\nbind_dn: {self.bind_dn}\nauth_pass: {self.auth_pass}\n{self.state}'




from enum import Enum, auto

class State(Enum):
    """States to manage client."""
    Connected = auto()
    Signed = auto()
    Disconnected = auto()

    def __str__(self) -> str:
        return f'state: {self.name}'




import ldap
from typing import Any, NoReturn


class LdapSuccessResult:
    """Ldap Success Result."""

    def __init__(self, payload: Any):
        self.payload = payload

    def unwrap(self) -> Any:
        return self.payload

class LdapErrorResult(Exception):
    """Ldap Error Result."""
    
    def __init__(self, message):
        self.message = message

    def unwrap(self) -> NoReturn:
        raise self


# ref: https://stackoverflow.com/questions/11420464/catch-exceptions-inside-a-class
def catch_exception(f):
    """Function decorator exception handler!"""
    def func(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except ldap.LDAPError as e:
            if args[0].debug:
                raise e
            
            return LdapErrorResult(e)
    return func




from typing import Any
import logging

from ..client.error_handler import LdapErrorResult
from ..logger import logger


def match_level(message: Any, level: int = 10) -> None:
    """Match level of loggin and send message!"""
    match level:
        case 10:
            logger.debug(message)
        case 20:
            logger.info(message)
        case 30:
            logger.warning(message)
        case 40:
            logger.error(message)
        case 50:
            logger.critical(message)
        case _:
            logger.debug(f"counld not match any level for this message: {message}")

class ADSuccessResult:
    """Active Directory Success Result."""

    def __init__(self, payload: Any):
        self.payload = payload

    def unwrap(self, level: int = logging.INFO) -> Any:
        """Get payload, send log maintaing the flexibility to change the loggin level of success."""
        if isinstance(self.payload, str):
            match_level(self.payload, level)
        return self.payload

class ADErrorResult(Exception):
    """Active Directory Error Result."""
    
    def __init__(self, message, fn: str):
        self.message = f'{fn}: {message}'

    def unwrap(self, level: int = logging.ERROR) -> str:
        """Get message, send log maintaing the flexibility to change the loggin level of the exception."""
        match_level(self.message, level)
        return self.message
   
# ref: https://stackoverflow.com/questions/11420464/catch-exceptions-inside-a-class
def catch_exception(f):
    """Function decorator exception handler."""
    def func(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except (LdapErrorResult, AssertionError) as e:
            return ADErrorResult(e, f.__name__)
    return func



import os
from ..msad import MsAD, AccountControlCode as Acc
from typing import Dict
from dotenv import dotenv_values
from typing import Dict, Optional

# Local Active Directory Settings

def load_env_variables_from_file(file_name: str) -> Dict[str, Optional[str]]:
    """Load envinroment variables from file using dotenv module."""
    return dotenv_values(file_name)

def load_env_variables() -> Dict[str, Optional[str]]:
    """Load envinroment variables from host environment variables."""
    return {
        "URI" : os.environ.get("URI"),
        "BASE_DN": os.environ.get("BASE_DN"),
        "BIND_DN": os.environ.get("BIND_DN"),
        "AUTH_PASS": os.environ.get("AUTH_PASS"),
        "CA_PATH": os.environ.get("CA_PATH")
    }

def ensure_loaded_variables(config: Dict[str, Optional[str]]) -> Dict[str, str]:
    """Verify if a item is None and modify they with empty string."""
    d = {}
    for k,v in config.items():
        if v is not None:
            d[f"{k}"] = v
        else:
            d[f"{k}"] = ""
    return d 

def create_user(name: str, passwd: str, config: Dict[str, str], acc: Acc, debug = False):
    """Create user in ms ad server."""
    ldap = MsAD(config["URI"], config["BASE_DN"], config["BIND_DN"], config["AUTH_PASS"], debug)
    ldap.start_tls(config["CA_PATH"]).unwrap()
    ldap.connect().unwrap()
    ldap.create_user(name, passwd, acc).unwrap()
    ldap.close().unwrap()

def create_user_from_ldif(from_ldif: str, config: Dict[str, str], debug = False):
    """Create user in ms ad server from ldif file."""
    ldap = MsAD(config["URI"], config["BASE_DN"], config["BIND_DN"], config["AUTH_PASS"], debug)
    ldap.start_tls(config["CA_PATH"]).unwrap()
    ldap.connect().unwrap()
    ldap.create_user_from_ldif(from_ldif).unwrap()
    ldap.close().unwrap()

def add_account_to_group(name: str, group_dn: str, config: Dict[str, str], debug = False):
    """Add a user account to a group of Active directory server."""
    ldap = MsAD(config["URI"], config["BASE_DN"], config["BIND_DN"], config["AUTH_PASS"], debug)
    ldap.start_tls(config["CA_PATH"]).unwrap()
    ldap.connect().unwrap()
    s_filter = f"(&(objectClass=User)(sAMAccountName={name}))"
    ldap.add_account_to_group(s_filter, group_dn).unwrap()
    ldap.close().unwrap()

def remove_account_from_group(name: str, group_dn: str, config: Dict[str, str], debug = False):
    """Remove user account from a group of Active directory server."""
    ldap = MsAD(config["URI"], config["BASE_DN"], config["BIND_DN"], config["AUTH_PASS"], debug)
    ldap.start_tls(config["CA_PATH"]).unwrap()
    ldap.connect().unwrap()
    s_filter = f"(&(objectClass=User)(sAMAccountName={name}))"
    ldap.remove_account_from_group(s_filter, group_dn).unwrap()
    ldap.close().unwrap()

def modify_acc(name: str, acc: Acc, config: Dict[str, str], debug = False):
    """Enable/Disable user account in ms ad server."""
    ldap = MsAD(config["URI"], config["BASE_DN"], config["BIND_DN"], config["AUTH_PASS"], debug)
    ldap.start_tls(config["CA_PATH"]).unwrap()
    ldap.connect().unwrap()
    s_filter = f"(&(objectClass=User)(sAMAccountName={name}))"
    ldap.modify_account_control(s_filter, acc).unwrap()
    ldap.close().unwrap()

def delete_account(name: str, config: Dict[str, str], debug = False):
    """Delete user account in ms ad server."""
    ldap = MsAD(config["URI"], config["BASE_DN"], config["BIND_DN"], config["AUTH_PASS"], debug)
    ldap.start_tls(config["CA_PATH"]).unwrap()
    ldap.connect().unwrap()
    ldap.delete_user(name).unwrap()
    ldap.close().unwrap()




from pathlib import Path
import logging
import logging.config

settings_path = Path.joinpath(Path(__file__).parent.parent, Path('config/logging_settings.ini'))
print(settings_path)
logging.config.fileConfig(settings_path)
logger = logging.getLogger('root')



import unittest
from padc.msad import MsAD
from padc.msad import AccountControlCode as Acc
from padc.msad.ad_error_handler import ADSuccessResult
from padc.cli.users_utils import load_env_variables_from_file, ensure_loaded_variables


class MsADTest(unittest.TestCase):
    """Active Directory unittest class."""

    def setUp(self):
        """Load needed env variables."""
        self.config = ensure_loaded_variables(load_env_variables_from_file(".env"))
        self.uri = self.config["URI"]
        self.base_dn = self.config["BASE_DN"]
        self.bind_dn = self.config["BIND_DN"]
        self.auth_pass = self.config["AUTH_PASS"]
        self.ca_path = self.config["CA_PATH"] 

        self.ldap = MsAD(self.uri, self.base_dn, self.bind_dn, self.auth_pass, False)
        self.new_user = "unittester pythonic"
        self.new_passwd = "unittestPassword@123"
        self.filter = '(&(objectClass=User)(sAMAccountName=unittester))'

    def test_connect_to_ad_server(self):
        """Should connect to ad server!"""
        tls = self.ldap.start_tls(self.ca_path)
        self.assertTrue(isinstance(tls, ADSuccessResult))
        print(tls.unwrap())

        conn = self.ldap.connect()
        self.assertTrue(isinstance(conn, ADSuccessResult))
        print(conn.unwrap())

        close = self.ldap.close()
        self.assertTrue(isinstance(close, ADSuccessResult))
        print(close.unwrap())
 
    def test_create_account(self):
        """Should create a new user in ad server!"""
        self.ldap.start_tls(self.ca_path)
        self.ldap.connect()

        user = self.ldap.create_user(self.new_user, self.new_passwd, Acc.NormalAccount)
        self.assertTrue(isinstance(user, ADSuccessResult))
        print(user.unwrap())

        self.ldap.close()

    def test_disable_account(self):
        """Should disable a user account by modify userAccountCode!"""
        self.ldap.start_tls(self.ca_path)
        self.ldap.connect()

        mod = self.ldap.modify_account_control(self.filter, Acc.DisableAccount)
        self.assertTrue(isinstance(mod, ADSuccessResult))
        print(mod.unwrap())

        self.ldap.close()

    def test_enable_account(self):
        """Should enable a user account by modify userAccountCode!"""
        self.ldap.start_tls(self.ca_path)
        self.ldap.connect()

        mod = self.ldap.modify_account_control(self.filter, Acc.NormalAccount)
        self.assertTrue(isinstance(mod, ADSuccessResult))
        print(mod.unwrap())        

        self.ldap.close()

  
    def test_delete_account(self):
        """Should delete a user account!"""
        self.ldap.start_tls(self.ca_path)
        self.ldap.connect()

        user = self.ldap.delete_user(self.new_user)
        self.assertTrue(isinstance(user, ADSuccessResult))
        print(user.unwrap())

        self.ldap.close()




if __name__ == '__main__':
    unittest.main()



import typer
from .users import users

# Create cli and Registry subcommands
app = typer.Typer(help="Python active directory user management cli!", pretty_exceptions_show_locals=False)
app.add_typer(users, name="users")

def run():
    app()



import typer
from .users_utils import *
from ..msad import match_code, AccountControlCode as Acc
from pathlib import Path

users = typer.Typer(help="Users subcommand")

@users.command("create-ldif")
def cli_create_from_ldif(
        from_file: Optional[Path] = typer.Option(
            None, 
            "--file",
            "-f",
            help="A .env file or load from host envvars by ignoring this argument",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        from_ldif: Optional[Path] = typer.Option(
            ..., 
            "--ldif",
            "-l",
            help="A ldif file to create a new user",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        debug: bool = typer.Option(False, help="Enable debug mode")
        ):
    """Create a new user account from ldif in Microsoft Active Directory Server."""
    config = {}
    if from_file:
        config = load_env_variables_from_file(str(from_file))
    else:
        config = load_env_variables()

    config = ensure_loaded_variables(config)

    create_user_from_ldif(str(from_ldif), config, debug)

@users.command("create")
def cli_create_user(
        from_file: Optional[Path] = typer.Option(
            None, 
            "--file",
            "-f",
            help="A .env file or load from host envvars by ignoring this argument",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        name: str = typer.Argument(..., help="A CN attribute value, don't pass a single name"), 
        password: str = typer.Argument(..., help="A user account password, unicodePwd will be create"),
        account_control_code: int = typer.Option(
            512,
            "--acc",
            "-c",
            help="User Account Control Code, 512 is NormalAccount and 514 is DisableAccount",
            show_default=True
            ),
        debug: bool = typer.Option(False, help="Enable debug mode")
        ):

    """Create a new user account in Microsoft Active Directory Server."""
    config = {}
    if from_file:
        config = load_env_variables_from_file(str(from_file))
    else:
        config = load_env_variables()

    config = ensure_loaded_variables(config)

    create_user(name, password, config, match_code(account_control_code), debug)

@users.command("delete")
def cli_delete_user(
        from_file: Optional[Path] = typer.Option(
            None, 
            "--file",
            "-f",
            help="A .env file or load from host envvars by ignoring this argument",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        name: str = typer.Argument(..., help="A CN attribute value, don't pass a single name"),
        debug: bool = typer.Option(False, help="Enable debug mode")
        ):

    """Delete a user account in Microsoft Active Directory Server."""
    config = {}
    if from_file:
        config = load_env_variables_from_file(str(from_file))
    else:
        config = load_env_variables()

    config = ensure_loaded_variables(config)

    delete_account(name, config, debug)

@users.command("disable")
def cli_disable_user(
        from_file: Optional[Path] = typer.Option(
            None, 
            "--file",
            "-f",
            help="A .env file or load from host envvars by ignoring this argument",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        name: str = typer.Argument(..., help="A sAMAccountName attribute value, don't pass CN "), 
        debug: bool = typer.Option(False, help="Enable debug mode")
        ):

    """Disable a user account in Microsoft Active Directory Server."""
    config = {}
    if from_file:
        config = load_env_variables_from_file(str(from_file))
    else:
        config = load_env_variables()

    config = ensure_loaded_variables(config)

    modify_acc(name, Acc.DisableAccount, config, debug)

@users.command("enable")
def cli_enable_user(
        from_file: Optional[Path] = typer.Option(
            None, 
            "--file",
            "-f",
            help="A .env file or load from host envvars by ignoring this argument",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        name: str = typer.Argument(..., help="A sAMAccountName attribute value, don't pass CN "), 
        debug: bool = typer.Option(False, help="Enable debug mode")
        ):

    """Enable a user account in Microsoft Active Directory Server."""
    config = {}
    if from_file:
        config = load_env_variables_from_file(str(from_file))
    else:
        config = load_env_variables()

    config = ensure_loaded_variables(config)

    modify_acc(name, Acc.NormalAccount, config, debug) 

@users.command("add-to-group")
def cli_add_to_group(
        from_file: Optional[Path] = typer.Option(
            None, 
            "--file",
            "-f",
            help="A .env file or load from host envvars by ignoring this argument",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        name: str = typer.Argument(..., help="A sAMAccountName attribute value, don't pass CN "), 
        groupdn: str = typer.Argument(..., help="A group DN attribute value"),
        debug: bool = typer.Option(False, help="Enable debug mode")
        ):

    """Add a user account to group in Microsoft Active Directory Server."""
    config = {}
    if from_file:
        config = load_env_variables_from_file(str(from_file))
    else:
        config = load_env_variables()

    config = ensure_loaded_variables(config)

    add_account_to_group(name, groupdn, config, debug) 

@users.command("remove-from-group")
def cli_remove_from_group(
        from_file: Optional[Path] = typer.Option(
            None, 
            "--file",
            "-f",
            help="A .env file or load from host envvars by ignoring this argument",
            exists=True,
            file_okay=True,
            dir_okay=False,
            writable=False,
            readable=True,
            resolve_path=True,
        ),
        name: str = typer.Argument(..., help="A sAMAccountName attribute value, don't pass CN "), 
        groupdn: str = typer.Argument(..., help="A group DN attribute value"),
        debug: bool = typer.Option(False, help="Enable debug mode")
        ):

    """Remove user account from group in Microsoft Active Directory Server."""
    config = {}
    if from_file:
        config = load_env_variables_from_file(str(from_file))
    else:
        config = load_env_variables()

    config = ensure_loaded_variables(config)

    remove_account_from_group(name, groupdn, config, debug) 



import sys, ldap
from typing import List, Dict, Tuple, Optional, Any
from enum import Enum

from ldap.modlist import modifyModlist, addModlist
from ldap.dn import str2dn
from ldif import LDIFRecordList
from .state import State
from .error_handler import catch_exception, LdapSuccessResult

class Scope(Enum):
    """Search Scope interface."""
    Base = ldap.SCOPE_BASE
    SubTree = ldap.SCOPE_SUBTREE

    def __repr__(self) -> str:
        return f'Scope: {self.name}'

class Client:
    """Base class to manipulate LDAP connections!"""
   
    @catch_exception
    def __init__(self, uri: str, log_level: int = 0, debug: bool = False) -> None:
        """Initialize a new LDAP object."""

        self.debug = debug
        self.connection = ldap.initialize(uri, log_level, sys.stderr)
        self.connection.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
        self.connection.set_option(ldap.OPT_REFERRALS, 0)

        if self.debug:
            self.connection.set_option(ldap.OPT_DEBUG_LEVEL, 255)

        self.state = State.Connected

    @catch_exception
    def _start_tls(self, ca_path: Optional[str] = None) -> LdapSuccessResult:
        """Start tls connection by passing a ca-certificate path."""

        assert self.state == State.Connected, "Cannot start tls without initialized connection"

        if ca_path:
            self.connection.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND)
            self.connection.set_option(ldap.OPT_X_TLS_CACERTFILE, ca_path)

        self.connection.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
        self.connection.start_tls_s()
        return LdapSuccessResult("_start_tls: Started tls ldap server connection")

    @catch_exception
    def _search(self, base: str, s_filter: str,  attr: Optional[List[str]], scope: Scope = Scope.SubTree) -> LdapSuccessResult:
        """Perform LDAP search on target DN, if attributes are None all objects will return."""

        assert self.state == State.Signed, "Cannot perform search without a signed connection"
        entry = self.connection.search_s(base, scope.value, s_filter, attr, 0)
        return LdapSuccessResult(entry)
   
    @catch_exception
    def _modify_add(self, target_dn: str, entry: Dict[str, Any]) -> LdapSuccessResult:
        """Modify entry by modlist add operation on it at target_dn."""
        assert self.state == State.Signed, "Cannot perform modify without a signed connection"
        modlist = [ (ldap.MOD_ADD, k, v) for k,v in entry.items() ]
        self.connection.modify_s(target_dn, modlist)
        return LdapSuccessResult(f"_modify_add: {target_dn}")

    @catch_exception
    def _modify_delete(self, target_dn: str, entry: Dict[str, Any]) -> LdapSuccessResult:
        """Modify entry by modlist delete operation on it at target_dn."""
        assert self.state == State.Signed, "Cannot perform modify without a signed connection"
        modlist = [ (ldap.MOD_DELETE, k, v) for k,v in entry.items() ]
        self.connection.modify_s(target_dn, modlist)
        return LdapSuccessResult(f"_modify_delete: {target_dn}")

    @catch_exception
    def _modify_replace(self, target_dn: str, entry: Tuple[Dict[str, Any], Dict[str, Any]]) -> LdapSuccessResult:
        """Modify entries by modlist replace (delete and add) operation on it based on argument entry."""

        assert self.state == State.Signed, "Cannot perform modify without a signed connection"
        old,new = entry
        modlist = modifyModlist(old, new)
        
        self.connection.modify_s(target_dn, modlist)
        return LdapSuccessResult(f"_modify_replace: {target_dn}")

    @catch_exception
    def _add(self, target_dn: str, entry: Dict[str, Any]) -> LdapSuccessResult:
        """Create a new entry at target DN."""
        
        assert self.state == State.Signed, "Cannot perform add without a signed connection"

        modlist = addModlist(entry)
        self.connection.add_s(target_dn, modlist)
        return LdapSuccessResult(f"_add: {target_dn}")

    @catch_exception
    def _delete(self, target_dn: str) -> LdapSuccessResult:
        """Delete entry based on target DN."""
        assert self.state == State.Signed, "Cannot perform delete without signed connection"

        self.connection.delete_s(target_dn)
        return LdapSuccessResult(f"_delete: {target_dn}")

    @catch_exception
    def _parse_dn(self, target_dn: str) -> LdapSuccessResult:
        """Parse target DN to sepated elements."""
        return LdapSuccessResult(str2dn(target_dn, ldap.DN_FORMAT_LDAPV3))

    @catch_exception
    def _parse_ldif(self, ldif_path: str) -> LdapSuccessResult:
        """Parse a ldif file, return all records inside payload."""
        parser = LDIFRecordList(open(ldif_path, "r"))
        parser.parse()
        return LdapSuccessResult(parser.all_records)

    @catch_exception
    def _bind(self, new_bind: str, new_pass: str) -> LdapSuccessResult:
        """Bind to connected server."""

        assert self.state == State.Connected, "Cannot perform bind without connection"

        self.connection.simple_bind_s(new_bind, new_pass)
        self.state = State.Signed

        return LdapSuccessResult(f"_bind: {new_bind}")

    @catch_exception
    def _close(self) -> LdapSuccessResult:
        """Close connection with LDAP server, turn connection object invalid."""

        assert self.state == State.Signed, "Cannot close connection without a connection"
        self.connection.unbind_s()
        self.state = State.Disconnected

        return LdapSuccessResult("_close: Unbinded connection in ldap server")




from setuptools import setup
import setuptools

setuptools.find_packages()

with open("README.md", 'r') as f:
    long_description = f.read()

with open("LICENSE", 'r') as f:
    l = f.read()

setup(
    name="padc",
    version="0.3",
    description="python active directory cli using python-ldap lib to manage ad servers",
    author="Fabio Kleis",
    author_email="fabiohkrc@gmail.com",
    url="https://github.com/Fabiokleis/p_ad",
    license=l,
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_data={'padc': ['config/logging_settings.ini',]},
    install_requires=[
        'future==0.18.3',
        'pyad==0.6.0',
        'pyasn1==0.4.8',
        'pyasn1-modules==0.2.8',
        'python-dotenv==0.21.1',
        'python-ldap==3.4.3',
        'click==8.1.3',
        'colorama==0.4.6',
        'commonmark==0.9.1',
        'Pygments==2.14.0',
        'rich==12.6.0',
        'shellingham==1.5.0.post1',
        'typer==0.7.0'
    ],
    entry_points={
        'console_scripts': ['padc=padc.cli.__main__:main']
    }
)
